/*
 * ${license.title}
 * Copyright (C) 2022 ${license.owner}
 * ${license.mailto}
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

//
// Created by webpigeon on 22/04/22.
//

#ifndef FGGL_DEMO_GAMESCENE_H
#define FGGL_DEMO_GAMESCENE_H

#include <fggl/app.hpp>
#include <fggl/gui/gui.hpp>

#include <fggl/data/heightmap.h>
#include <fggl/data/procedural.hpp>

#include <fggl/gfx/camera.hpp>
#include <fggl/input/input.hpp>
#include <fggl/input/camera_input.h>

#include <fggl/util/service.h>
#include <fggl/util/chrono.hpp>

#include <PerlinNoise.hpp>

enum camera_type { cam_free, cam_arcball };

class GameScene : public fggl::AppState {
	public:
		explicit GameScene(fggl::App& app) : fggl::AppState(app), m_world(nullptr), m_sceneTime(nullptr), m_inputs(nullptr) { };
		//~GameScene() override = default;

		void activate() override {
			auto& locator = fggl::util::ServiceLocator::instance();
			m_inputs = locator.providePtr<fggl::input::Input>();

			// setup the world using the global type registry
			// FIXME: type registry probably doesn't need to be application-global - will make savegame/mod support complicated
			m_world = std::make_unique<fggl::ecs3::World>( *m_owner.registry() );

			m_sceneTime = std::make_unique<fggl::util::Timer>();
			m_sceneTime->frequency( glfwGetTimerFrequency() );
			m_sceneTime->setup( glfwGetTimerValue() );

			setup();
		}

		void setup();
		void update() override;
		void render(fggl::gfx::Graphics& gfx) override;

	private:
		std::unique_ptr<fggl::ecs3::World> m_world;
		std::unique_ptr<fggl::util::Timer> m_sceneTime;
		std::shared_ptr<fggl::input::Input> m_inputs;
		fggl::gui::Panel m_canvas;
};


#endif //FGGL_DEMO_GAMESCENE_H
