#include <filesystem>
#include <iostream>
#include <memory>

#include <fggl/app.hpp>

#include <fggl/gfx/atlas.hpp>
#include <fggl/gfx/window.hpp>
#include "fggl/gfx/compat.hpp"
#include "fggl/gfx/ogl/compat.hpp"

#include <fggl/data/storage.hpp>
#include <fggl/util/service.h>

#include <fggl/ecs3/types.hpp>

#include "fggl/scenes/menu.hpp"
#include "GameScene.h"

// prototype of resource discovery
void discover(const std::filesystem::path& base) {

	std::vector< std::filesystem::path > contentPacks;
	for ( const auto& item : std::filesystem::directory_iterator(base) ) {

		// content pack detection
		if ( std::filesystem::is_directory( item ) ) {
			auto manifest = item.path() / "manifest.yml";
			if ( std::filesystem::exists( manifest ) ) {
				contentPacks.push_back( item.path() );
			}
		}
		
	}

	// what did we find?
	std::cerr << "found pack(s): " << std::endl;
	for ( auto& pack : contentPacks ) {
		std::cerr << pack << std::endl;
	}

}

static void test_atlas_api() {
	// atlas testing
	std::vector< fggl::gfx::ImageAtlas<char>::SubImage > images;
	auto *atlas = fggl::gfx::ImageAtlas<char>::pack(images);
}

int main(int argc, const char* argv[]) {
    fggl::App app( "fggl-demo" );

    // FIXME: janky API(s)
    auto& locator = fggl::util::ServiceLocator::instance();
	auto inputs = locator.supply<fggl::input::Input>(std::make_shared<fggl::input::Input>());
	auto storage = locator.supply<fggl::data::Storage>(std::make_shared<fggl::data::Storage>());

	locator.supply<fggl::gui::FontLibrary>(std::make_shared<fggl::gui::FontLibrary>());
	locator.supply<fggl::ecs3::TypeRegistry>(std::make_shared<fggl::ecs3::TypeRegistry>());

    // Would be nice to not take args like this, it messes with lifetimes
    auto& windowing = app.use<fggl::gfx::ecsGlfwModule>(inputs);

    // -- should not be our problem - this is a broken api
    auto window = windowing.createWindow("Demo Game");
    window->make_graphics<fggl::gfx::OpenGL4>();
    window->fullscreen( true );
    app.setWindow( std::move(window) );

	// load a bunch of modules to provide game functionality
	//app.use<fggl::ecs3::ecsTypes>();
	app.use<fggl::gfx::SceneUtils>();

	test_atlas_api();

	// Add a basic main menu
    auto *menu = app.add_state<fggl::scenes::BasicMenu>("menu");
    menu->add("start", [&app]() { app.change_state("game"); });
	menu->add("options", [&app]() { app.change_state("game"); });
	menu->add("quit", [&app]() { app.running(false); });

	// the game state itself
    app.add_state<GameScene>("game");

	return app.run(argc, argv);
}
